@@ -474,19 +474,27 @@ static void signal_plugins(int sig)
 	}
 }
 
-static int write_to_plugin(event_t *e, lnode *conf)
+static int write_to_plugin(event_t *e, const char *string, size_t string_len,
+			   lnode *conf)
 {
 	int rc;
-	struct iovec vec[2];
 
-	vec[0].iov_base = &e->hdr;
-	vec[0].iov_len = sizeof(struct audit_dispatcher_header);
+	if (conf->p->format == F_STRING) {
+		do {
+			rc = write(conf->p->plug_pipe[1], string, string_len);
+		} while (rc < 0 && errno == EINTR);
+	} else {
+		struct iovec vec[2];
 
-	vec[1].iov_base = e->data;
-	vec[1].iov_len = MAX_AUDIT_MESSAGE_LENGTH;
-	do {
-		rc = writev(conf->p->plug_pipe[1], vec, 2);
-	} while (rc < 0 && errno == EINTR);
+		vec[0].iov_base = &e->hdr;
+		vec[0].iov_len = sizeof(struct audit_dispatcher_header);
+
+		vec[1].iov_base = e->data;
+		vec[1].iov_len = MAX_AUDIT_MESSAGE_LENGTH;
+		do {
+			rc = writev(conf->p->plug_pipe[1], vec, 2);
+		} while (rc < 0 && errno == EINTR);
+	}
 	return rc;
 }
 
@@ -637,15 +645,7 @@ static int event_loop(void)
 					send_af_unix_binary(e);
 			} else if (conf->p->type == S_ALWAYS) {
 				int rc;
-				if (conf->p->format == F_STRING) {
-					do {
-						rc = write(
-							conf->p->plug_pipe[1],
-							v, len);
-					} while (rc < 0 && errno == EINTR);
-				} else {
-					rc = write_to_plugin(e, conf);
-				}
+				rc = write_to_plugin(e, v, len, conf);
 				if (rc < 0 && errno == EPIPE) {
 					/* Child disappeared ? */
 					syslog(LOG_ERR,
@@ -662,11 +662,13 @@ static int event_loop(void)
 					close(conf->p->plug_pipe[1]);
 					conf->p->plug_pipe[1] = -1;
 					conf->p->active = A_NO;
-					if (start_one_plugin(conf) == 0) {
-						rc = write_to_plugin(e, conf);
+					if (start_one_plugin(conf)) {
+						rc = write_to_plugin(e, v, len,
+								     conf);
 						syslog(LOG_NOTICE,
 						"plugin %s was restarted",
 							conf->p->path);
+						conf->p->active = A_YES;
 					} 
 				}
 			}
