@@ -179,7 +179,7 @@ auparse_state_t *auparse_init(ausource_t source, const void *b)
 			break;
 		case AUSOURCE_DESCRIPTOR:
 			n = (long)b;
-			au->in = fdopen(n, "r");
+			au->in = fdopen(n, "rm");
 			break;
 		case AUSOURCE_FILE_POINTER:
 			au->in = (FILE *)b;
@@ -686,7 +686,7 @@ static int extract_timestamp(const char *b, au_event_t *e)
 		// Optionally grab the node - may or may not be included
 		if (*ptr == 'n') {
 			e->host = strdup(ptr+5);
-			ptr = strtok(NULL, " "); // Bump along to the next one
+			(void)strtok(NULL, " "); // Bump along to the next one
 		}
 		// at this point we have type=
 		ptr = strtok(NULL, " ");
@@ -782,9 +782,13 @@ static int retrieve_next_line(auparse_state_t *au)
 		case AUSOURCE_FILE_ARRAY:
 			// if the first time through, open file
 			if (au->list_idx == 0 && au->in == NULL) {
+				if (au->source_list[au->list_idx] == NULL) {
+					errno = 0;
+					return -2;
+				}
 				au->line_number = 0;
 				au->in = fopen(au->source_list[au->list_idx],
-									"r");
+									"rm");
 				if (au->in == NULL)
 					return -1;
 				__fsetlocking(au->in, FSETLOCKING_BYCALLER);
@@ -802,7 +806,7 @@ static int retrieve_next_line(auparse_state_t *au)
 					if (au->source_list[au->list_idx]) {
 						au->in = fopen(
 						  au->source_list[au->list_idx],
-						  "r");
+						  "rm");
 						if (au->in == NULL)
 							return -1;
 						__fsetlocking(au->in,
