@@ -1,6 +1,6 @@
 /*
 * interpret.c - Lookup values to something more readable
-* Copyright (c) 2007-09 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2007-09,2011 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -74,6 +74,8 @@
 #include "socktabs.h"
 #include "seeks.h"
 #include "typetabs.h"
+#include "nfprototabs.h"
+#include "icmptypetabs.h"
 
 typedef enum { AVC_UNSET, AVC_DENIED, AVC_GRANTED } avc_t;
 typedef enum { S_UNSET=-1, S_FAILED, S_SUCCESS } success_t;
@@ -494,7 +496,7 @@ static const char *print_sockaddr(const char *val)
         const struct sockaddr *saddr;
         char name[NI_MAXHOST], serv[NI_MAXSERV];
         const char *host;
-        char *out;
+        char *out = NULL;
         const char *str;
 
         slen = strlen(val)/2;
@@ -943,6 +945,71 @@ static const char *print_signals(const char *val)
 	return out;
 }
 
+static const char *print_nfproto(const char *val)
+{
+        int proto;
+	char *out;
+	const char *s;
+
+        errno = 0;
+        proto = strtoul(val, NULL, 10);
+        if (errno) {
+                asprintf(&out, "conversion error(%s)", val);
+                return out;
+        }
+
+	s = nfproto_i2s(proto);
+	if (s != NULL)
+		return strdup(s);
+	asprintf(&out, "unknown netfilter protocol (%s)", val);
+	return out;
+}
+
+static const char *print_icmptype(const char *val)
+{
+        int icmptype;
+	char *out;
+	const char *s;
+
+        errno = 0;
+        icmptype = strtoul(val, NULL, 10);
+        if (errno) {
+                asprintf(&out, "conversion error(%s)", val);
+                return out;
+        }
+
+	s = icmptype_i2s(icmptype);
+	if (s != NULL)
+		return strdup(s);
+	asprintf(&out, "unknown icmp type (%s)", val);
+	return out;
+}
+
+static const char *print_protocol(const char *val)
+{
+	int i;
+	char *out;
+
+	errno = 0;
+        i = strtoul(val, NULL, 10);
+	if (errno) 
+		asprintf(&out, "conversion error(%s)", val);
+	else {
+		struct protoent *p = getprotobynumber(i);
+		if (p)
+			out = strdup(p->p_name);
+		else
+			out = strdup("undefined protocol");
+	}
+	return out;
+}
+
+static const char *print_addr(const char *val)
+{
+	char *out = strdup(val);
+	return out;
+}
+
 static const char *print_list(const char *val)
 {
 	int i;
@@ -1114,12 +1181,14 @@ const char *interpret(const rnode *r)
 	const char *val = nvlist_get_cur_val(nv);
 
 	/* Do some fixups */
-	if (r->type == AUDIT_EXECVE && name[0] == 'a')
+	if (r->type == AUDIT_EXECVE && name[0] == 'a' && strcmp(name, "argc"))
 		type = AUPARSE_TYPE_ESCAPED;
 	else if (r->type == AUDIT_AVC && strcmp(name, "saddr") == 0)
 		type = -1;
 	else if (r->type == AUDIT_USER_TTY && strcmp(name, "msg") == 0)
 		type = AUPARSE_TYPE_ESCAPED;
+	else if (r->type == AUDIT_NETFILTER_PKT && strcmp(name, "saddr") == 0)
+		type = AUPARSE_TYPE_ADDR;
 	else if (strcmp(name, "acct") == 0) {
 		if (val[0] == '"')
 			type = AUPARSE_TYPE_ESCAPED;
@@ -1194,6 +1263,18 @@ const char *interpret(const rnode *r)
 		case AUPARSE_TYPE_CAP_BITMAP:
 			out = print_cap_bitmap(val);
 			break;
+		case AUPARSE_TYPE_NFPROTO:
+			out = print_nfproto(val);
+			break; 
+		case AUPARSE_TYPE_ICMPTYPE:
+			out = print_icmptype(val);
+			break; 
+		case AUPARSE_TYPE_PROTOCOL:
+			out = print_protocol(val);
+			break; 
+		case AUPARSE_TYPE_ADDR:
+			out = print_addr(val);
+			break;
 		case AUPARSE_TYPE_UNCLASSIFIED:
 		default: {
 			char *out2;
