@@ -34,6 +34,7 @@
 #include <errno.h>
 #include <sys/poll.h>
 #include <sys/utsname.h>
+#include <sys/stat.h>
 #include <fcntl.h>	/* O_NOFOLLOW needs gnu defined */
 #include <limits.h>	/* for PATH_MAX */
 
@@ -243,7 +244,7 @@ static int load_libaudit_config(const char *path)
 	}
 
 	/* it's ok, read line by line */
-	f = fdopen(fd, "r");
+	f = fdopen(fd, "rm");
 	if (f == NULL) {
 		audit_msg(LOG_ERR, "Error - fdopen failed (%s)",
 			strerror(errno));
@@ -278,8 +279,10 @@ static int load_libaudit_config(const char *path)
 			lineno++;
 			continue;
 		}
-		if (nv.value == NULL)
-			return 1; 
+		if (nv.value == NULL) {
+			fclose(f);
+			return 1;
+		}
 
 		/* identify keyword or error */
 		kw = kw_lookup(nv.name);
@@ -1088,7 +1091,7 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 			}
 			break;
 		case AUDIT_FILETYPE:
-			if (flags != AUDIT_FILTER_EXIT && flags != AUDIT_FILTER_ENTRY)
+			if (!(flags == AUDIT_FILTER_EXIT || flags == AUDIT_FILTER_ENTRY))
 				return -17;
 			rule->values[rule->field_count] = 
 				audit_name_to_ftype(v);
@@ -1120,8 +1123,8 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 					return -13;
 			}
 
-			if (field == AUDIT_PPID && flags != AUDIT_FILTER_EXIT 
-				&& flags != AUDIT_FILTER_ENTRY)
+			if (field == AUDIT_PPID && !(flags == AUDIT_FILTER_EXIT
+				|| flags == AUDIT_FILTER_ENTRY))
 				return -17;
 			
 			if (flags == AUDIT_FILTER_EXCLUDE)
@@ -1130,7 +1133,12 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 			if (!isdigit((char)*(v)))
 				return -21;
 
-			rule->values[rule->field_count] = strtol(v, NULL, 0);
+			if (field == AUDIT_INODE)
+				rule->values[rule->field_count] =
+					strtoul(v, NULL, 0);
+			else
+				rule->values[rule->field_count] =
+					strtol(v, NULL, 0);
 			break;
 	}
 	rule->field_count++;
