@@ -1,5 +1,5 @@
 /* libaudit.h -- 
- * Copyright 2004-2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2011 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -53,7 +53,8 @@ extern "C" {
  * 2200 - 2299 user space actions taken in response to anomalies
  * 2300 - 2399 user space generated LSPP events
  * 2400 - 2499 user space crypto events
- * 2500 - 2999 future user space (maybe integrity labels and related events)
+ * 2500 - 2599 user space virtualization management events
+ * 2600 - 2999 future user space (maybe integrity labels and related events)
  */
 
 #define AUDIT_FIRST_USER_MSG    1100    /* First user space message */
@@ -88,6 +89,8 @@ extern "C" {
 #define AUDIT_SYSTEM_BOOT	1127	/* System boot */
 #define AUDIT_SYSTEM_SHUTDOWN	1128	/* System shutdown */
 #define AUDIT_SYSTEM_RUNLEVEL	1129	/* System runlevel change */
+#define AUDIT_SERVICE_START	1130	/* Service (daemon) start */
+#define AUDIT_SERVICE_STOP	1131	/* Service (daemon) stop */
 
 #define AUDIT_FIRST_DAEMON	1200
 #define AUDIT_LAST_DAEMON	1299
@@ -191,14 +194,35 @@ extern "C" {
 #define AUDIT_CRYPTO_KEY_USER		2404 /* Create,delete,negotiate */
 #define AUDIT_CRYPTO_FAILURE_USER	2405 /* Fail decrypt,encrypt,randomiz */
 #define AUDIT_CRYPTO_REPLAY_USER	2406 /* Crypto replay detected */
+#define AUDIT_CRYPTO_SESSION		2407 /* Record parameters set during
+						TLS session establishment */
 
 #define AUDIT_LAST_CRYPTO_MSG		2499
 
+#define AUDIT_FIRST_VIRT_MSG		2500
+#define AUDIT_VIRT_CONTROL		2500 /* Start, Pause, Stop VM */
+#define AUDIT_VIRT_RESOURCE		2501 /* Resource assignment */
+#define AUDIT_VIRT_MACHINE_ID		2502 /* Binding of label to VM */
+
+#define AUDIT_LAST_VIRT_MSG		2599
+
 #ifndef AUDIT_FIRST_USER_MSG2
 #define AUDIT_FIRST_USER_MSG2  2100    /* More userspace messages */
 #define AUDIT_LAST_USER_MSG2   2999
 #endif
 
+/* New kernel event definitions since 2.6.30 */
+#ifndef AUDIT_MMAP
+#define AUDIT_MMAP		1323 /* Descriptor and flags in mmap */
+#endif
+
+#ifndef AUDIT_NETFILTER_PKT
+#define AUDIT_NETFILTER_PKT	1324 /* Packets traversing netfilter chains */
+#endif
+#ifndef AUDIT_NETFILTER_CFG
+#define AUDIT_NETFILTER_CFG	1325 /* Netfilter chain modifications */
+#endif
+
 
 /* This is related to the filterkey patch */
 #define AUDIT_KEY_SEPARATOR 0x01
