@@ -1,5 +1,5 @@
 /* auditctl.c -- 
- * Copyright 2004-2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2011 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -424,12 +424,14 @@ static int equiv_parse(char *optarg, char **mp, char **sub)
 	return 0;
 }
 
-void audit_request_rule_list(int fd)
+int audit_request_rule_list(int fd)
 {
 	if (audit_request_rules_list_data(fd) > 0) {
 		list_requested = 1;
 		get_reply();
+		return 1;
 	}
+	return 0;
 }
 
 void check_rule_mismatch(int lineno, const char *option)
@@ -489,6 +491,7 @@ static int setopt(int count, int lineno, char *vars[])
 		break;
 	case 'i':
 		ignore = 1;
+		retval = -2;
 		break;
         case 's':
 		retval = audit_request_status(fd);
@@ -582,8 +585,10 @@ static int setopt(int count, int lineno, char *vars[])
 				break;
 			}
 		}
-		audit_request_rule_list(fd);
-		retval = -2;
+		if (audit_request_rule_list(fd))
+			retval = -2;
+		else
+			retval = -1;
 		break;
         case 'a':
 		if (strstr(optarg, "task") && audit_syscalladded) {
@@ -772,7 +777,7 @@ static int setopt(int count, int lineno, char *vars[])
 		}
 		retval = delete_all_rules(fd);
 		if (retval == 0) {
-			audit_request_rule_list(fd);
+			(void)audit_request_rule_list(fd);
 			key[0] = 0;
 			retval = -2;
 		}
@@ -983,7 +988,7 @@ static int fileopt(const char *file)
 		return 1;
 	}
 
-        f = fdopen(tfd, "r");
+        f = fdopen(tfd, "rm");
         if (f == NULL) {
                 fprintf(stderr, "Error - fdopen failed (%s)\n",
                         strerror(errno));
@@ -1040,7 +1045,7 @@ static int fileopt(const char *file)
 					fclose(f);
 					return 0;
 				}
-				if (!ignore) {
+				if (ignore == 0) {
 					fclose(f);
 					return -1;
 				}
@@ -1075,7 +1080,7 @@ int main(int argc, char *argv[])
 		if (audit_is_enabled(fd) == 2) {
 			fprintf(stderr,
 				"The audit system is in immutable "
-				"mode, no rules loaded\n");
+				"mode, no rule changes allowed\n");
 			return 0;
 		} else if (errno == ECONNREFUSED) {
 			fprintf(stderr, "The audit system is disabled\n");
@@ -1101,7 +1106,7 @@ int main(int argc, char *argv[])
 		if (audit_is_enabled(fd) == 2) {
 			fprintf(stderr,
 				"The audit system is in immutable "
-				"mode, no rules loaded\n");
+				"mode, no rule changes allowed\n");
 			free(rule_new);
 			return 0;
 		} else if (errno == ECONNREFUSED) {
@@ -1110,7 +1115,9 @@ int main(int argc, char *argv[])
 			return 0;
 		}
 	}
-	return handle_request(retval);
+	retval = handle_request(retval);
+	free(rule_new);
+	return retval;
 }
 
 /*
@@ -1396,7 +1403,13 @@ static int audit_print_reply(struct audit_reply *rep)
 							strcat(perms, "a");
 						printf(" perm=%s", perms);
 						show_syscall = 0;
+					} else if (field == AUDIT_INODE) {
+						// Unsigned items
+						printf(" %s%s%u", name, 
+							audit_operator_to_symbol(op),
+							rep->ruledata->values[i]);
 					} else {
+						// Signed items
 						printf(" %s%s%d", name, 
 							audit_operator_to_symbol(op),
 							rep->ruledata->values[i]);
