@@ -1,5 +1,5 @@
 /* aureport-options.c - parse commandline options and configure aureport
- * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08,2010-11 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ const char *event_terminal = NULL;
 const char *event_subject = NULL;
 const char *event_object = NULL;
 int event_exit = 0, event_exit_is_set = 0;
-int event_ppid = -1, event_session_id = -1;
+int event_ppid = -1, event_session_id = -2;
 
 /* These are used by aureport */
 const char *dummy = "dummy";
@@ -150,7 +150,7 @@ static void usage(void)
 {
 	printf("usage: aureport [options]\n"
 	"\t-a,--avc\t\t\tAvc report\n"
-	"\t--auth\t\t\t\tAuthentication report\n"
+	"\t-au,--auth\t\t\tAuthentication report\n"
 	"\t-c,--config\t\t\tConfig change report\n"
 	"\t-cr,--crypto\t\t\tCrypto report\n"
 	"\t-e,--event\t\t\tEvent report\n"
@@ -176,6 +176,7 @@ static void usage(void)
 	"\t-te,--end [end date] [end time]\tending date & time for reports\n"
 	"\t-tm,--terminal\t\t\tTerMinal name report\n"
 	"\t-ts,--start [start date] [start time]\tstarting data & time for reports\n"
+	"\t--tty\t\t\t\tReport about tty keystrokes\n"
 	"\t-u,--user\t\t\tUser name report\n"
 	"\t-v,--version\t\t\tVersion\n"
 	"\t-x,--executable\t\t\teXecutable name report\n"
