@@ -1,6 +1,6 @@
 /*
 * ausearch-parse.c - Extract interesting fields and check for match
-* Copyright (c) 2005-08 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-08, 2011 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -45,14 +45,17 @@ static int common_path_parser(search_items *s, char *path);
 static int avc_parse_path(const lnode *n, search_items *s);
 static int parse_path(const lnode *n, search_items *s);
 static int parse_user(const lnode *n, search_items *s);
+static int parse_obj(const lnode *n, search_items *s);
 static int parse_login(const lnode *n, search_items *s);
-static int parse_daemon(const lnode *n, search_items *s);
+static int parse_daemon1(const lnode *n, search_items *s);
+static int parse_daemon2(const lnode *n, search_items *s);
 static int parse_sockaddr(const lnode *n, search_items *s);
 static int parse_avc(const lnode *n, search_items *s);
 static int parse_integrity(const lnode *n, search_items *s);
 static int parse_kernel_anom(const lnode *n, search_items *s);
 static int parse_simple_message(const lnode *n, search_items *s);
 static int parse_tty(const lnode *n, search_items *s);
+static int parse_pkt(const lnode *n, search_items *s);
 
 
 static int audit_avc_init(search_items *s)
@@ -104,12 +107,20 @@ int extract_search_items(llist *l)
 			case AUDIT_LOGIN:
 				ret = parse_login(n, s);
 				break;
+			case AUDIT_OBJ_PID:
+				ret = parse_obj(n, s);
+				break;
 			case AUDIT_DAEMON_START:
 			case AUDIT_DAEMON_END:
 			case AUDIT_DAEMON_ABORT:
 			case AUDIT_DAEMON_CONFIG:
 			case AUDIT_DAEMON_ROTATE:
-				ret = parse_daemon(n, s);
+			case AUDIT_DAEMON_RESUME:
+				ret = parse_daemon1(n, s);
+				break;
+			case AUDIT_DAEMON_ACCEPT:
+			case AUDIT_DAEMON_CLOSE:
+				ret = parse_daemon2(n, s);
 				break;
 			case AUDIT_CONFIG_CHANGE:
 				ret = parse_simple_message(n, s);
@@ -117,6 +128,9 @@ int extract_search_items(llist *l)
 			case AUDIT_AVC:
 				ret = parse_avc(n, s);
 				break;
+			case AUDIT_NETFILTER_PKT:
+				ret = parse_pkt(n, s);
+				break;
 			case
 			   AUDIT_FIRST_KERN_ANOM_MSG...AUDIT_LAST_KERN_ANOM_MSG:
 				ret = parse_kernel_anom(n, s);
@@ -130,12 +144,16 @@ int extract_search_items(llist *l)
 			case AUDIT_KERNEL:
 			case AUDIT_IPC:
 			case AUDIT_SELINUX_ERR:
+			case AUDIT_EXECVE:
+			case AUDIT_BPRM_FCAPS:
+			case AUDIT_NETFILTER_CFG:
 				// Nothing to parse
 				break;
 			case AUDIT_TTY:
 				ret = parse_tty(n, s);
 				break;
 			default:
+				// printf("unparsed type:%d\n", n->type);
 				break;
 			}
 			// if (ret) printf("type:%d ret:%d\n", n->type, ret);
@@ -348,7 +366,7 @@ static int parse_syscall(lnode *n, search_items *s)
 		}
 	}
 	// ses
-	if (event_session_id != -1 ) {
+	if (event_session_id != -2 ) {
 		str = strstr(term, "ses=");
 		if (str) {
 			ptr = str + 4;
@@ -507,7 +525,7 @@ static int common_path_parser(search_items *s, char *path)
 		//create
 		s->filename = malloc(sizeof(slist));
 		if (s->filename == NULL)
-			return 4;
+			return 1;
 		slist_create(s->filename);
 	}
 	if (*path == '"') {
@@ -515,7 +533,7 @@ static int common_path_parser(search_items *s, char *path)
 		path++;
 		term = strchr(path, '"');
 		if (term == NULL)
-			return 1;
+			return 2;
 		*term = 0;
 		if (s->filename) {
 			// append
@@ -548,7 +566,7 @@ static int common_path_parser(search_items *s, char *path)
 				goto append;
 			}
 			if (!isxdigit(path[0]))
-				return 5;
+				return 4;
 			if (path[0] == '0' && path[1] == '0')
 				sn.str = unescape(&path[2]); // Abstract name
 			else
@@ -558,7 +576,7 @@ static int common_path_parser(search_items *s, char *path)
 				(sn.str[1] == '/')) && s->cwd) {
 				char *tmp = malloc(PATH_MAX);
 				if (tmp == NULL)
-					return 6;
+					return 5;
 				snprintf(tmp, PATH_MAX, "%s/%s", 
 					s->cwd, sn.str);
 				free(sn.str);
@@ -628,6 +646,34 @@ static int parse_path(const lnode *n, search_items *s)
 	return 0;
 }
 
+static int parse_obj(const lnode *n, search_items *s)
+{
+	char *str, *term;
+
+	term = n->message;
+	if (event_object) {
+		// obj context
+		str = strstr(term, "obj=");
+		if (str != NULL) {
+			str += 4;
+			term = strchr(str, ' ');
+			if (term)
+				*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.tcontext = strdup(str);
+				alist_append(s->avc, &an);
+				if (term)
+					*term = ' ';
+			} else
+				return 1;
+		}
+	}
+	return 0;
+}
+
 static int parse_user(const lnode *n, search_items *s)
 {
 	char *ptr, *str, *term, saved, *mptr;
@@ -684,7 +730,7 @@ static int parse_user(const lnode *n, search_items *s)
 		return 9;
 	*term = ' ';
 	// ses
-	if (event_session_id != -1 ) {
+	if (event_session_id != -2 ) {
 		str = strstr(term, "ses=");
 		if (str) {
 			ptr = str + 4;
@@ -706,7 +752,7 @@ static int parse_user(const lnode *n, search_items *s)
 			str += 5;
 			term = strchr(str, ' ');
 			if (term == NULL)
-				return 10;
+				return 12;
 			*term = 0;
 			if (audit_avc_init(s) == 0) {
 				anode an;
@@ -716,7 +762,7 @@ static int parse_user(const lnode *n, search_items *s)
 				alist_append(s->avc, &an);
 				*term = ' ';
 			} else
-				return 11;
+				return 13;
 		}
 	}
 	// get uid - something has uid after auid ??
@@ -727,18 +773,36 @@ static int parse_user(const lnode *n, search_items *s)
 		while (isdigit(*term))
 			term++;
 		if (term == ptr)
-			return 12;
+			return 14;
 
 		saved = *term;
 		*term = 0;
 		errno = 0;
 		s->uid = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 13;
+			return 15;
 		*term = saved;
 	}
 	mptr = term + 1;
 
+	if (event_comm) {
+		// dont do this search unless needed
+		str = strstr(mptr, "comm=");
+		if (str) {
+			str += 5;
+			if (*str == '"') {
+				str++;
+				term = strchr(str, '"');
+				if (term == NULL)
+					return 16;
+				*term = 0;
+				s->comm = strdup(str);
+				*term = '"';
+			} else 
+				s->comm = unescape(str);
+		}
+	}
+
 	// Get acct for user/group add/del
 	str = strstr(mptr, "acct=");
 	if (str != NULL) {
@@ -785,7 +849,7 @@ static int parse_user(const lnode *n, search_items *s)
 			if (term == NULL) {
 				term = strchr(str, ' ');
 				if (term == NULL)
-					return 15;
+					return 17;
 			}
 			saved = *term;
 			*term = 0;
@@ -803,7 +867,7 @@ static int parse_user(const lnode *n, search_items *s)
 					if (term == NULL) {
 						term = strchr(str, ' ');
 						if (term == NULL)
-							return 16;
+							return 18;
 					}
 					saved = *term;
 					*term = 0;
@@ -823,7 +887,7 @@ static int parse_user(const lnode *n, search_items *s)
 			if (term == NULL) {
 				term = strchr(str, ')');
 				if (term == NULL)
-					return 17;
+					return 19;
 			}
 			*term = 0;
 			s->terminal = strdup(str);
@@ -839,7 +903,7 @@ static int parse_user(const lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 18;
+					return 20;
 				*term = 0;
 				s->exe = strdup(str);
 				*term = '"';
@@ -873,7 +937,7 @@ static int parse_user(const lnode *n, search_items *s)
 			ptr = str + 4;
 			term = strchr(ptr, '\'');
 			if (term == NULL)
-				return 19;
+				return 21;
 			*term = 0;
 			if (strncmp(ptr, "failed", 6) == 0)
 				s->success = S_FAILED;
@@ -884,7 +948,7 @@ static int parse_user(const lnode *n, search_items *s)
 			ptr = str + 7;
 			term = strchr(ptr, ')');
 			if (term == NULL)
-				return 20;
+				return 22;
 			*term = 0;
 			if (strcasecmp(ptr, "success") == 0)
 				s->success = S_SUCCESS;
@@ -967,7 +1031,9 @@ static int parse_login(const lnode *n, search_items *s)
 			s->success = S_SUCCESS; 
 	}
 	// ses
-	if (event_session_id != -1 ) {
+	if (event_session_id != -2 ) {
+		if (term == NULL)
+			term = n->message;
 		str = strstr(term, "new ses=");
 		if (str) {
 			ptr = str + 8;
@@ -985,7 +1051,7 @@ static int parse_login(const lnode *n, search_items *s)
 	return 0;
 }
 
-static int parse_daemon(const lnode *n, search_items *s)
+static int parse_daemon1(const lnode *n, search_items *s)
 {
 	char *ptr, *str, *term, saved, *mptr;
 
@@ -997,34 +1063,34 @@ static int parse_daemon(const lnode *n, search_items *s)
 	// auid
 	str = strstr(mptr, "auid=");
 	if (str == NULL)
-		return 2;
+		return 1;
 	ptr = str + 5;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 3;
+		return 2;
 	saved = *term;
 	*term = 0;
 	errno = 0;
 	s->loginuid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 4;
+		return 3;
 	*term = saved;
 
 	// pid
 	if (event_pid != -1) {
 		str = strstr(term, "pid=");
 		if (str == NULL)
-			return 5;
+			return 4;
 		ptr = str + 4;
 		term = strchr(ptr, ' ');
 		if (term == NULL) 
-			return 6;
+			return 5;
 		saved = *term;
 		*term = 0;
 		errno = 0;
 		s->pid = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 7;
+			return 6;
 		*term = saved;
 	}
 
@@ -1035,7 +1101,7 @@ static int parse_daemon(const lnode *n, search_items *s)
 			str += 5;
 			term = strchr(str, ' ');
 			if (term == NULL)
-				return 8;
+				return 7;
 			*term = 0;
 			if (audit_avc_init(s) == 0) {
 				anode an;
@@ -1045,7 +1111,7 @@ static int parse_daemon(const lnode *n, search_items *s)
 				alist_append(s->avc, &an);
 				*term = ' ';
 			} else
-				return 9;
+				return 8;
 		}
 	}
 
@@ -1057,7 +1123,53 @@ static int parse_daemon(const lnode *n, search_items *s)
 			while (isalpha(*term))
 				term++;
 			if (term == ptr)
-				return 10;
+				return 9;
+			saved = *term;
+			*term = 0;
+			if (strncmp(ptr, "failed", 6) == 0)
+				s->success = S_FAILED;
+			else
+				s->success = S_SUCCESS;
+			*term = saved;
+		}
+	}
+
+	return 0;
+}
+
+static int parse_daemon2(const lnode *n, search_items *s)
+{
+	char *str, saved, *term = n->message;
+
+	if (event_hostname) {
+		str = strstr(term, "addr=");
+		if (str) {
+			str += 5;
+			term = strchr(str, ':');
+			if (term == NULL) {
+				term = strchr(str, ' ');
+				if (term == NULL)
+					return 1;
+			}
+			saved = *term;
+			*term = 0;
+			free(s->hostname);
+			s->hostname = strdup(str);
+			*term = saved;
+		} else
+			term = n->message; 
+	}
+
+	if (event_success != S_UNSET) {
+		char *str = strstr(term, "res=");
+		if (str) {
+			char *ptr, *term, saved;
+
+			ptr = term = str + 4;
+			while (isalpha(*term))
+				term++;
+			if (term == ptr)
+				return 2;
 			saved = *term;
 			*term = 0;
 			if (strncmp(ptr, "failed", 6) == 0)
@@ -1173,12 +1285,12 @@ static int parse_integrity(const lnode *n, search_items *s)
 		ptr = str + 4;
 		term = strchr(ptr, ' ');
 		if (term == NULL)
-			return 2;
+			return 1;
 		*term = 0;
 		errno = 0;
 		s->pid = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 3;
+			return 2;
 		*term = ' ';
 	}
 
@@ -1188,12 +1300,12 @@ static int parse_integrity(const lnode *n, search_items *s)
 		ptr = str + 4;
 		term = strchr(ptr, ' ');
 		if (term == NULL)
-			return 4;
+			return 3;
 		*term = 0;
 		errno = 0;
 		s->uid = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 5;
+			return 4;
 		*term = ' ';
 	}
 
@@ -1203,12 +1315,12 @@ static int parse_integrity(const lnode *n, search_items *s)
 		ptr = str + 5;
 		term = strchr(ptr, ' ');
 		if (term == NULL)
-			return 6;
+			return 5;
 		*term = 0;
 		errno = 0;
 		s->loginuid = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 7;
+			return 6;
 		*term = ' ';
 	}
 
@@ -1219,7 +1331,7 @@ static int parse_integrity(const lnode *n, search_items *s)
 			str++;
 			term = strchr(str, '"');
 			if (term == NULL)
-				return 8;
+				return 7;
 			*term = 0;
 			s->comm = strdup(str);
 			*term = '"';
@@ -1231,7 +1343,7 @@ static int parse_integrity(const lnode *n, search_items *s)
 	if (str) {
 		str += 6;
 		if (common_path_parser(s, str))
-			return 9;
+			return 8;
 	}
 
 	// and results (usually last)
@@ -1244,7 +1356,7 @@ static int parse_integrity(const lnode *n, search_items *s)
 		errno = 0;
 		s->success = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 10;
+			return 9;
 		if (term)
 			*term = ' ';
 	}
@@ -1432,7 +1544,7 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 		term = n->message;
 
 	// get uid
-	str = strstr(term, " uid="); // if promiscuous, we start over
+	str = strstr(term, "uid="); // if promiscuous, we start over
 	if (str) {
 		ptr = str + 4;
 		term = strchr(ptr, ' ');
@@ -1461,6 +1573,22 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 		*term = ' ';
 	}
 
+	str = strstr(term, "ses=");
+	if (str) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term)
+			*term = 0;
+		errno = 0;
+		s->session_id = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 7;
+		if (term)
+			*term = ' ';
+		else
+			term = ptr;
+	}
+
 	if (event_subject) {
 		// scontext
 		str = strstr(term, "subj=");
@@ -1468,7 +1596,7 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 			str += 5;
 			term = strchr(str, ' ');
 			if (term == NULL)
-				return 7;
+				return 8;
 			*term = 0;
 			if (audit_avc_init(s) == 0) {
 				anode an;
@@ -1478,7 +1606,7 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 				alist_append(s->avc, &an);
 				*term = ' ';
 			} else
-				return 8;
+				return 9;
 		}
 	}
 
@@ -1489,12 +1617,12 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 			ptr = str + 4;
 			term = strchr(ptr, ' ');
 			if (term == NULL)
-				return 9;
+				return 10;
 			*term = 0;
 			errno = 0;
 			s->pid = strtoul(ptr, NULL, 10);
 			if (errno)
-				return 10;
+				return 11;
 			*term = ' ';
 		}
 	}
@@ -1508,7 +1636,7 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 11;
+					return 12;
 				*term = 0;
 				s->comm = strdup(str);
 				*term = '"';
@@ -1546,19 +1674,21 @@ static int parse_simple_message(const lnode *n, search_items *s)
 	}
 
 	// ses
-	if (event_session_id != -1 ) {
+	if (event_session_id != -2 ) {
 		str = strstr(term, "ses=");
 		if (str) {
 			ptr = str + 4;
 			term = strchr(ptr, ' ');
-			if (term == NULL)
-				return 3;
-			*term = 0;
+			if (term)
+				*term = 0;
 			errno = 0;
 			s->session_id = strtoul(ptr, NULL, 10);
 			if (errno)
-				return 4;
-			*term = ' ';
+				return 3;
+			if (term)
+				*term = ' ';
+			else
+				term = ptr;
 		}
 	}
 
@@ -1582,7 +1712,7 @@ static int parse_simple_message(const lnode *n, search_items *s)
 				else	// Set it back to something sane
 					term = str;
 			} else
-				return 5;
+				return 4;
 		}
 	}
 
@@ -1593,7 +1723,7 @@ static int parse_simple_message(const lnode *n, search_items *s)
 				//create
 				s->key = malloc(sizeof(slist));
 				if (s->key == NULL)
-					return 6;
+					return 5;
 				slist_create(s->key);
 			}
 			ptr = str + 4;
@@ -1612,7 +1742,7 @@ static int parse_simple_message(const lnode *n, search_items *s)
 					}
 					*term = '"';
 				} else
-					return 7;
+					return 6;
 			} else {
 				if (s->key) {
 					char *saved=NULL;
@@ -1652,7 +1782,7 @@ static int parse_simple_message(const lnode *n, search_items *s)
 			errno = 0;
 			s->success = strtoul(ptr, NULL, 10);
 			if (errno)
-				return 8;
+				return 7;
 			if (term)
 				*term = ' ';
 		}
@@ -1711,9 +1841,11 @@ static int parse_tty(const lnode *n, search_items *s)
 		return 6;
 	if (term)
 		*term = ' ';
+	else
+		term = ptr;
 
 	// ses
-	if (event_session_id != -1 ) {
+	if (event_session_id != -2 ) {
 		str = strstr(term, "ses=");
 		if (str) {
 			ptr = str + 4;
@@ -1771,3 +1903,45 @@ static int parse_tty(const lnode *n, search_items *s)
 	return 0;
 }
 
+static int parse_pkt(const lnode *n, search_items *s)
+{
+	char *str, *ptr, *term=n->message;
+
+	// get hostname
+	if (event_hostname) {
+		str = strstr(n->message, "saddr=");
+		if (str) {
+			ptr = str + 6;
+			term = strchr(ptr, ' ');
+			if (term == NULL)
+				return 1;
+			*term = 0;
+			s->hostname = strdup(ptr);
+			*term = ' ';
+		}
+	}
+
+	// obj context
+	if (event_object) {
+		str = strstr(term, "obj=");
+		if (str != NULL) {
+			str += 4;
+			term = strchr(str, ' ');
+			if (term)
+				*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.tcontext = strdup(str);
+				alist_append(s->avc, &an);
+				if (term)
+					*term = ' ';
+			} else
+				return 2;
+		}
+	}
+
+	return 0;
+}
+
