@@ -1,6 +1,6 @@
 /*
  * ausearch.c - main file for ausearch utility 
- * Copyright 2005-08 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-08,2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -225,7 +225,7 @@ static int process_stdin(void)
 
 static int process_file(char *filename)
 {
-	log_fd = fopen(filename, "r");
+	log_fd = fopen(filename, "rm");
 	if (log_fd == NULL) {
 		fprintf(stderr, "Error opening %s (%s)\n", filename, 
 			strerror(errno));
@@ -281,7 +281,8 @@ static int get_record(llist **l)
 			}
 		} else {
 			free(buff);
-			if ((ferror(log_fd) && errno == EINTR)||feof(log_fd)) {
+			if ((ferror_unlocked(log_fd) &&
+			     errno == EINTR) || feof_unlocked(log_fd)) {
 				terminate_all_events(&lo);
 				*l = get_ready_event(&lo);
 				if (*l)
