@@ -1,6 +1,6 @@
 /*
  * aulast.c - A last program based on audit logs 
- * Copyright (c) 2008-2009 Red Hat Inc., Durham, North Carolina.
+ * Copyright (c) 2008-2009,2011 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This software may be freely redistributed and/or modified under the
@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <locale.h>
 #include <string.h>
+#include <unistd.h>
 #include <errno.h>
 #include <pwd.h>
 #include <stdlib.h>
@@ -68,7 +69,10 @@ static void report_session(lnode* cur)
 		}
 	} else {
 		struct passwd *p = getpwuid(cur->auid);
-		printf("%-8.8s ", p->pw_name);
+		if (p)
+			printf("%-8.8s ", p->pw_name);
+		else
+			printf("%-8.u ", cur->auid);
 	}
 	if (strncmp("/dev/", cur->term, 5) == 0)
 		printf("%-12.12s ", cur->term+5);
@@ -203,7 +207,7 @@ static void create_new_session(auparse_state_t *au)
 
 static void update_session_login(auparse_state_t *au)
 {
-	const char *tpid, *tses, *tuid, *tacct, *host, *term, *tres;
+	const char *tpid, *tses, *tuid, *tacct=NULL, *host, *term, *tres;
 	int pid = -1, uid = -1, ses = -1, result = -1;
 	time_t start;
 	lnode *cur;
@@ -297,10 +301,11 @@ static void update_session_login(auparse_state_t *au)
 				auparse_get_serial(au));
 
 		// If the results were failed, we can close it out
+		/* FIXME: result cannot be true. This is dead code.
 		if (result) {
 			report_session(cur);
 			list_delete_cur(&l);
-		} 
+		} */
 	} else if (bad == 1 && result == 1) {
 		// If it were a bad login and we are wanting bad logins
 		// create the record and report it.
@@ -505,10 +510,14 @@ int main(int argc, char *argv[])
 		au = auparse_init(AUSOURCE_FILE, file);
 	else if (use_stdin)
 		au = auparse_init(AUSOURCE_FILE_POINTER, stdin);
-	else
+	else {
+		if (getuid()) {
+			fprintf(stderr, "You probably need to be root for this to work\n");
+		}
 		au = auparse_init(AUSOURCE_LOGS, NULL);
+	}
 	if (au == NULL) {
-		printf("Error - %s\n", strerror(errno));
+		fprintf(stderr, "Error - %s\n", strerror(errno));
 		goto error_exit_1;
 	}
 
